
#include <windows.h>  
#include <commctrl.h>
#include "delcol.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "ListView_DeleteColumn()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

// Return TRUE if a column exists.
//................................
BOOL ColumnExists( HWND hList, int nCol )
{
   LV_COLUMN lvc;

   // Retrieve the width of a column to determine if it exists.
   //..........................................................
   lvc.mask = LVCF_WIDTH;

   return( ListView_GetColumn( hList, nCol, &lvc ) );
}

// Determine the number of columns in the list view.
//..................................................
int GetNumColumns( HWND hList )
{
   int nNum = 0;

   while( ColumnExists( hList, nNum ) )
      nNum++;

   return( nNum );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HWND hList = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              {
                 InitCommonControls();

                 // Create the list view.
                 //......................
                 hList  = CreateWindowEx( 0, WC_LISTVIEW, "",
                                          WS_CHILD | WS_VISIBLE | LVS_REPORT,
                                          10, 100, 100, 100,
                                          hWnd,
                                          (HMENU)1,
                                          hInst,
                                          NULL);
              }
              break;

      case WM_SIZE :
              if ( hList )
                 MoveWindow( hList, 0, 0, LOWORD( lParam ), HIWORD( lParam ), FALSE );

              if ( wParam == SIZE_RESTORED || wParam == SIZE_MAXIMIZED )
                 ListView_RedrawItems( hList, 0, ListView_GetItemCount( hList ) );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_NEWCOLUMN :
                        {
                           LV_COLUMN col;
                           char      szName[40];
                           int       nNumCols = GetNumColumns( hList );

                           // Add a new column to the list view.
                           //...................................
                           wsprintf( szName, "Column %d", nNumCols+1 );

                           col.mask     = LVCF_FMT | LVCF_SUBITEM | 
                                          LVCF_WIDTH | LVCF_TEXT;
                           col.fmt      = LVCFMT_LEFT;
                           col.cx       = 100;
                           col.pszText  = szName;
                           col.iSubItem = nNumCols;

                           ListView_InsertColumn( hList, nNumCols, &col );
                        }
                        break;

                 case IDM_RENCOL :
                 case IDM_RENFIELD :
                        {
                           LV_COLUMN col;
                           int       nCol = 0;
                           char      szName[40];

                           while ( ColumnExists( hList, nCol ) )
                           {
                              col.mask    = LVCF_TEXT;
                              col.pszText = szName;
                              
                              if ( LOWORD( wParam ) == IDM_RENCOL )
                                 wsprintf( szName, "Column %d", nCol+1 );
                              else
                                 wsprintf( szName, "Field %d", nCol+1 );

                              ListView_SetColumn( hList, nCol, &col );

                              nCol++;
                           }
                        }
                        break;

                 case IDM_DELETE :
                        // Delete the last column from the list view.
                        //...........................................
                        ListView_DeleteColumn( hList, GetNumColumns( hList )-1 );
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
